// Disclaimer:
// This software is provided AS IS for educational purposes only,
// with no warranties, and confers no rights. Error checking is
// elided for clarity. Carry out sufficient tests prior to using
// it in a productive environment.
// CASIO Europe GmbH accepts no responsibility for loss or damage
// arising from its use, including damage from virus.
//
// Please, note: This configuration information may be changed
// without any further notice.
// Casio Europe GmbH does not guarantee future compatibility. By
// using this software you agree to these terms and conditions.
// Casio IT-G6x0 ScanDemoIntent example 17.07.2020
// This Demo explains how to receive the scanner result with User Message Intent
////////////////////////////////////////////////////////////////////////////////

package casio.de.example.scannerintent;

import android.annotation.SuppressLint;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.graphics.Bitmap;
import android.os.Bundle;
import android.os.CountDownTimer;
import android.view.KeyEvent;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.MotionEvent;
import android.view.View;
import android.widget.ImageView;

import androidx.annotation.Nullable;
import androidx.appcompat.app.AppCompatActivity;

import com.google.zxing.BarcodeFormat;
import com.google.zxing.EncodeHintType;
import com.google.zxing.MultiFormatWriter;
import com.google.zxing.common.BitMatrix;
import com.google.zxing.qrcode.decoder.ErrorCorrectionLevel;
import com.journeyapps.barcodescanner.BarcodeEncoder;

import java.util.HashMap;
import java.util.Map;

import casio.de.example.scannerintent.databinding.ActivityMainBinding;
import jp.casio.ht.devicelibrary.ScannerLibrary;

import static java.lang.String.valueOf;

public class MainActivity extends AppCompatActivity {

    private static ScannerLibrary mScanLib; //ScannerLib instance
    private static ScannerLibrary.ScanResult mScanResult; //Result instance for USER MESSAGE

    private static ActivityMainBinding binding; //View binding object
    private static int TRIGGERKEYLEFT = 1011; //Left scanner key for orderbook scanning
    private static int TRIGGERKEYRIGHT = 1012; //Right scanner key for orderbook scanning
    private static int TRIGGERKEYCENTER = 1010; //Center scanner key for orderbook scanning
    private static int ORDERBOOKSCANNING_TIMEOUT = 10; //milliseconds
    private static int ORDERBOOKSCANNING_DETECTIONAREA = 3; // (0 - 9 is the range)
    private ScanResultReceiverUser myScanResultReceiverUser; //Local broadcastreceiver
    private IntentFilter intentFilter; //Intentfilter for RECEIVEACTION
    private String RECEIVEACTION = "device.common.USERMSG"; //Intent from scanner wedge

    /**
     * Get scanner lib instance
     *
     * @return scanner lib object
     */
    private static ScannerLibrary getScanLib() {
        if (mScanLib == null) {
            mScanLib = new ScannerLibrary();
        }
        return mScanLib;
    }

    /**
     * Get scan result instance
     *
     * @return scan result object
     */
    private static ScannerLibrary.ScanResult getScanLibResult() {
        if (mScanResult == null) {
            mScanResult = new ScannerLibrary.ScanResult();
        }
        return mScanResult;
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        binding = ActivityMainBinding.inflate(getLayoutInflater());
        View view = binding.getRoot();
        setContentView(view);

        binding.scanBtn.setOnTouchListener((v, event) -> { //Softbutton for scanner
            switch (event.getAction()) {
                case MotionEvent.ACTION_DOWN:
                    if (binding.orderSwitch.isChecked()) { //Only when orderscanning is enabled
                        getScanLib().turnAimerOn(ScannerLibrary.CONSTANT.LASER_HIGHLIGHT_MODE.ENABLE); //Enable Aimer
                    } else {
                        getScanLib().setTriggerKeyOn(ScannerLibrary.CONSTANT.TRIGGERKEY.ENABLE);
                    }
                    break;
                case MotionEvent.ACTION_UP:
                    if (binding.orderSwitch.isChecked()) { //Only when orderscanning is enabled
                        getScanLib().turnAimerOn(ScannerLibrary.CONSTANT.LASER_HIGHLIGHT_MODE.DISABLE); //Disable aimer
                        getScanLib().setTriggerKeyEnable(ScannerLibrary.CONSTANT.TRIGGERKEY.ENABLE); //Enable Triggerkey -> otherwise setTriggerKeyOn has no effect!
                        getScanLib().setTriggerKeyOn(ScannerLibrary.CONSTANT.TRIGGERKEY.ENABLE); //Start scanning

                        new CountDownTimer(ORDERBOOKSCANNING_TIMEOUT, 1) { //If there is no barcode in orderbook scanning, stop after 10ms

                            public void onTick(long millisUntilFinished) {
                                //not in use
                            }

                            public void onFinish() {
                                getScanLib().setTriggerKeyOn(ScannerLibrary.CONSTANT.TRIGGERKEY.DISABLE); //Stop after timeout
                                getScanLib().setTriggerKeyEnable(ScannerLibrary.CONSTANT.TRIGGERKEY.DISABLE); //Disable Triggerkey
                            }
                        }.start();
                    } else {
                        getScanLib().setTriggerKeyOn(ScannerLibrary.CONSTANT.TRIGGERKEY.DISABLE);
                    }
                    v.performClick();
                    break;
            }
            return false;
        });

        binding.orderSwitch.setChecked(false);
        binding.aimerSwitch.setOnCheckedChangeListener((buttonView, isChecked) -> changeLightMode());
        binding.illuminationSwitch.setOnCheckedChangeListener((buttonView, isChecked) -> changeLightMode());
        binding.orderSwitch.setOnCheckedChangeListener((buttonView, isChecked) -> changeScanMode());// Enable  / disable orderbook scanning
        binding.resultEd.setShowSoftInputOnFocus(false); //Disable ime keyboard
        binding.imageView.setScaleType(ImageView.ScaleType.CENTER_INSIDE); //Zxing barcode image result

        intentFilter = new IntentFilter();
        intentFilter.addAction(RECEIVEACTION);
        myScanResultReceiverUser = new ScanResultReceiverUser();
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        MenuInflater inflater = getMenuInflater();
        inflater.inflate(R.menu.menu, menu);
        return super.onCreateOptionsMenu(menu);
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            case R.id.menu_exit: //Exit application
                finish();
                return true;

            case R.id.menu_settings: //Open scanner settings
                Intent intent = new Intent();
                intent.setPackage("device.settings.scanner");
                startActivity(intent);
                return true;
            default:
                return super.onOptionsItemSelected(item);
        }
    }

    @Override
    public boolean dispatchKeyEvent(KeyEvent event) {
        int code = event.getKeyCode(); //Get the keycode

        if (event.getAction() == KeyEvent.ACTION_DOWN) {
            if (code == TRIGGERKEYLEFT || code == TRIGGERKEYRIGHT || code == TRIGGERKEYCENTER) {  //On key down -> start orderscanning
                if (binding.orderSwitch.isChecked()) { //Only when orderscanning is enabled
                    getScanLib().turnAimerOn(ScannerLibrary.CONSTANT.LASER_HIGHLIGHT_MODE.ENABLE); //Enable Aimer
                }
            }
        }

        if (event.getAction() == KeyEvent.ACTION_UP) {
            if (code == TRIGGERKEYLEFT || code == TRIGGERKEYRIGHT || code == TRIGGERKEYCENTER) { //On key down -> end orderscanning
                if (binding.orderSwitch.isChecked()) { //Only when orderscanning is enabled
                    getScanLib().turnAimerOn(ScannerLibrary.CONSTANT.LASER_HIGHLIGHT_MODE.DISABLE); //Disable aimer
                    getScanLib().setTriggerKeyEnable(ScannerLibrary.CONSTANT.TRIGGERKEY.ENABLE); //Enable Triggerkey -> otherwise setTriggerKeyOn has no effect!
                    getScanLib().setTriggerKeyOn(ScannerLibrary.CONSTANT.TRIGGERKEY.ENABLE); //Start scanning

                    new CountDownTimer(ORDERBOOKSCANNING_TIMEOUT, 1) { //If there is no barcode in orderbook scanning, stop after 10ms

                        public void onTick(long millisUntilFinished) {
                            //not in use
                        }

                        public void onFinish() {
                            getScanLib().setTriggerKeyOn(ScannerLibrary.CONSTANT.TRIGGERKEY.DISABLE); //Stop after timeout
                            getScanLib().setTriggerKeyEnable(ScannerLibrary.CONSTANT.TRIGGERKEY.DISABLE); //Disable Triggerkey
                        }
                    }.start();
                }
            }
        }
        return super.dispatchKeyEvent(event);
    }

    /**
     * Handle the different Settings in orderbook scanning
     */
    private void changeScanMode() {
        if (binding.orderSwitch.isChecked()) {
            getScanLib().setTriggerKeyEnable(ScannerLibrary.CONSTANT.TRIGGERKEY.DISABLE); //Disable the triggerkey
            getScanLib().setCenteringWindow(ScannerLibrary.CONSTANT.CENTERING_WINDOW_MODE.ENABLE); //Enable Centerwindow mode
            getScanLib().setDetectionAreaSize(ORDERBOOKSCANNING_DETECTIONAREA); //Set detection area to a low value (0-9 is the range)
        } else {
            getScanLib().setTriggerKeyEnable(ScannerLibrary.CONSTANT.TRIGGERKEY.ENABLE); //enable the triggerkey
            getScanLib().setCenteringWindow(ScannerLibrary.CONSTANT.CENTERING_WINDOW_MODE.DISABLE); //Disable Centerwindow mode
        }
    }

    /**
     * Create scanner and scanresult instance and
     */
    public void initScanner() {
        getScanLib().setOutputType(ScannerLibrary.CONSTANT.OUTPUT.USER); //Set scanner output typ to User Message
        //TODO: Some other scanner configuration

        getScanLib().openScanner(); //open the scanner
        getLightModeforUI(); //Get status of the configuration and set UI
    }

    /**
     * OnStart -> Register the broadcast receiver
     */
    @Override
    protected void onStart() {
        initScanner(); //Init scanner sequence
        registerReceiver(myScanResultReceiverUser, intentFilter); //Register the receiver for the scanner result
        super.onStart();
    }

    /**
     * OnStop -> Release the broadcastreceiver
     */
    @Override
    protected void onStop() {
        unregisterReceiver(myScanResultReceiverUser); //Unregister the receiver for the scanner result
        super.onStop();
    }

    /**
     * OnDestroy -> close the scanner instance and release the scanner instance
     */
    public void onDestroy() {
        getScanLib().closeScanner(); //Close the scanner on application exit
        if (mScanLib != null) { // release the scanner instance
            mScanLib = null;
        }
        if (mScanResult != null) { // release the scanresult instance
            mScanResult = null;
        }
        super.onDestroy();
    }

    /**
     * Set lightmode depending on UI switches
     */
    public void changeLightMode() {
        boolean aimerON = binding.aimerSwitch.isChecked();
        boolean illuminationON = binding.illuminationSwitch.isChecked();

        if (!aimerON && !illuminationON) { //Both switches off
            getScanLib().setLightMode(ScannerLibrary.CONSTANT.LIGHT_MODE.ALL_OFF);
        }

        if (aimerON && illuminationON) { //Both switches on
            getScanLib().setLightMode(ScannerLibrary.CONSTANT.LIGHT_MODE.ALL_ON);
        }

        if (aimerON && !illuminationON) { //Only aimer is on
            getScanLib().setLightMode(ScannerLibrary.CONSTANT.LIGHT_MODE.AIMER_ON);
        }

        if (!aimerON && illuminationON) { //Only illumination is on
            getScanLib().setLightMode(ScannerLibrary.CONSTANT.LIGHT_MODE.ILLUMINATION_ON);
        }
    }

    /**
     * Set UI switches from lightmode status
     */
    public void getLightModeforUI() {
        switch (getScanLib().getLightMode()) { //Read actual lightmode and setup UI

            case ScannerLibrary.CONSTANT.LIGHT_MODE.AIMER_ON: { //Only aimer is on
                binding.aimerSwitch.setChecked(true);
                binding.illuminationSwitch.setChecked(false);
            }
            break;

            case ScannerLibrary.CONSTANT.LIGHT_MODE.ILLUMINATION_ON: { //Only illumination is on
                binding.aimerSwitch.setChecked(false);
                binding.illuminationSwitch.setChecked(true);
            }
            break;

            case ScannerLibrary.CONSTANT.LIGHT_MODE.ALL_ON: { //Both switches on
                binding.aimerSwitch.setChecked(true);
                binding.illuminationSwitch.setChecked(true);
            }
            break;

            case ScannerLibrary.CONSTANT.LIGHT_MODE.ALL_OFF: { //Both switches off
                binding.aimerSwitch.setChecked(false);
                binding.illuminationSwitch.setChecked(false);
            }
            break;
        }
    }

    /**
     * Get symbology for Zxing image
     *
     * @param symbologyID
     * @return
     */
    @Nullable
    private BarcodeFormat getBarcodeTyp(int symbologyID) {
        switch (symbologyID) {
            case ScannerLibrary.CONSTANT.SYMBOLOGY.EAN13:
                return BarcodeFormat.EAN_13;
            case ScannerLibrary.CONSTANT.SYMBOLOGY.EAN8:
                return BarcodeFormat.EAN_8;
            case ScannerLibrary.CONSTANT.SYMBOLOGY.AZTEC:
                return BarcodeFormat.AZTEC;
            case ScannerLibrary.CONSTANT.SYMBOLOGY.CODABAR:
                return BarcodeFormat.CODABAR;
            case ScannerLibrary.CONSTANT.SYMBOLOGY.CODE39:
                return BarcodeFormat.CODE_39;
            case ScannerLibrary.CONSTANT.SYMBOLOGY.CODE93:
                return BarcodeFormat.CODE_93;
            case ScannerLibrary.CONSTANT.SYMBOLOGY.CODE128:
                return BarcodeFormat.CODE_128;
            case ScannerLibrary.CONSTANT.SYMBOLOGY.GS1_128:
                return BarcodeFormat.CODE_128;
            case ScannerLibrary.CONSTANT.SYMBOLOGY.DATAMATRIX:
                return BarcodeFormat.DATA_MATRIX;
            case ScannerLibrary.CONSTANT.SYMBOLOGY.MAXICODE:
                return BarcodeFormat.MAXICODE;
            case ScannerLibrary.CONSTANT.SYMBOLOGY.ITF:
                return BarcodeFormat.ITF;
            case ScannerLibrary.CONSTANT.SYMBOLOGY.QR:
                return BarcodeFormat.QR_CODE;
            case ScannerLibrary.CONSTANT.SYMBOLOGY.UPCA:
                return BarcodeFormat.UPC_A;
            case ScannerLibrary.CONSTANT.SYMBOLOGY.UPCE:
                return BarcodeFormat.UPC_E;
            case ScannerLibrary.CONSTANT.SYMBOLOGY.PDF417:
                return BarcodeFormat.PDF_417;
            case ScannerLibrary.CONSTANT.SYMBOLOGY.GS1_DATABAR:
                return BarcodeFormat.RSS_14;

            default:
                return null; //in case of unsupported code
        }
    }

    /**
     * Create a 1D / 2D barcode with Zxzing library and return the bitmap
     *
     * @return
     */
    private Bitmap createBarcode(String codeData, BarcodeFormat barcodeFormat, int x, int y) {
        Bitmap bitmap = null;
        MultiFormatWriter multiFormatWriter = new MultiFormatWriter();

        Map<EncodeHintType, ErrorCorrectionLevel> hints = new HashMap<>();
        //hints.put(EncodeHintType.ERROR_CORRECTION, ErrorCorrectionLevel.H); // in case we want to use error correction

        BitMatrix bitMatrix;
        try {
            bitMatrix = multiFormatWriter.encode(codeData, barcodeFormat, x - 15, y - 10, hints);
            BarcodeEncoder barcodeEncoder = new BarcodeEncoder();
            bitmap = barcodeEncoder.createBitmap(bitMatrix);
        } catch (Exception e) { //In case of any exception (WriterExeption, FormatException, ChecksumException)
            e.printStackTrace();
        }
        return bitmap;
    }


    /**
     * Resultreceiver for scanner with output type = User Message
     */
    public class ScanResultReceiverUser extends BroadcastReceiver {
        @SuppressLint("SetTextI18n")
        public void onReceive(Context context, Intent intent) {
            getScanLib().getScanResult(getScanLibResult()); //Read the scanresult

            if (binding.orderSwitch.isChecked()) {
                getScanLib().setTriggerKeyEnable(ScannerLibrary.CONSTANT.TRIGGERKEY.DISABLE);
            }

            if (getScanLibResult().length > 0) { //Set results to the userinterface (UTF-8)
                binding.resultEd.setText(new String(getScanLibResult().value));
                binding.resultTypTv.setText(getScanLibResult().symbologyName);
                binding.resultCountTv.setText(valueOf(getScanLibResult().length));
                binding.resultTimeTv.setText(getScanLibResult().time + "ms");

                BarcodeFormat barcodeFormat = getBarcodeTyp(getScanLibResult().symbologyID);
                if (barcodeFormat != null) {
                    binding.imageView.setImageBitmap(createBarcode(binding.resultEd.getText().toString(),
                            getBarcodeTyp(getScanLibResult().symbologyID), binding.imageView.getWidth(), binding.imageView.getHeight()));
                } else {
                    binding.imageView.setImageDrawable(getDrawable(R.drawable.nosupport));
                }

            } else { //Clear userinterface in case of no result data
                binding.resultEd.setText("");
                binding.imageView.setImageDrawable(getDrawable(R.drawable.imageviewshape));
                binding.resultTypTv.setText("");
                binding.resultCountTv.setText("");
                binding.resultTimeTv.setText("");
            }
        }
    }
}
